/*
 * Expression.h
 *
 *  Created on: 25 mars 2015
 *      Author: michel
 */

#ifndef EXPRESSION_H_
#define EXPRESSION_H_

#include <string>
#include "Argument.h"

namespace modele {

class Valeur; // déclaration en avant car Valeur hérite d'Expression et calculer retourne une Valeur
class Reference;

/** La classe Expression représente la classe ABSTRAITE racine de la hiérachie des noeuds calculables de l'arbre
 * représentant l'expression d'une cellule. Les Plages de cellules ne sont pas calculables !
 * @author Michel Meynard
 * @version 1.0
 */

class Expression :  public virtual Argument {
public:
  /** constructeur d'expression
   *
   */
  Expression() {}

  bool estCalculable(){ return true; }

  /** Méthode virtuelle pure retournant une chaîne représentant l'expression (arbre)
   * @return la chaîne
   */
  virtual std::string toString()=0;  // virtuelle pure(=0)

  /** Méthode virtuelle pure calculant la valeur de l'expression (arbre)
   * @return la valeur qui est un new objet et devra donc être delete.
   */
  virtual modele::Valeur * calculer()=0;  // virtuelle pure(=0), le retour doit être par réf (ou ptr) car on ne peut ret une classe abstraite
  /** Clonage arborescent d'une expression.
   *
   */
  virtual Expression *clone()=0;
  /** destructeur virtuel impur : seul le destructeur de l'hérité sera appelé
   */

  virtual void majRefRelatives(Reference *r){
    // ne rien faire sauf pour les ref relatives et relLignes et relCol et Exp ayant des sous-expressions (binaires, fons, ...) : surcharge
    // dans les fonctions traiter les plages : Argument non calculable
  }

  virtual ~Expression(){};
};

} /* namespace modele */

#endif /* EXPRESSION_H_ */
