/*
 * Feuille.h
 *
 *  Created on: 25 mars 2015
 *      Author: michel
 */

#ifndef FEUILLE_H_
#define FEUILLE_H_

#include <string>   // pour std::string et to_string()
#include <sstream>
#include <iostream>
#include <map>

#include "Cellule.h"
//#include "Reference.h" // une Reference contient un attribut Feuille !

using namespace std;

namespace modele {
class Reference;


/** La classe Feuille permet de rassembler des cellules.
 * La méthode principale est setCellule(...) qui permet de créer ou de
 * mettre à jour une cellule existante.
 * @author Michel Meynard
 * @version 1.0
 */
class Feuille {
  /** lors de la construction d'une cellule, celle-ci est ajoutée à sa feuille
   *
   */
  friend class Cellule;
private:
  static int nbFeuille; // nombre de Feuilles créées (doit être initialisé dans le cpp)

  std::string nom;    // Nom de la feuille
  /** La valeur est un ptr afin de maîtriser la création des Cellueles
   *
   */
  std::map<Reference, Cellule*> cellules; //=new map(); // Table associative des (Ref,cellule)

public:
  /** Constructeur de Feuille par défaut affectant des noms Feuille1, Feuille2, ...
   */
  Feuille();

  /** Constructeur de Feuille avec un nom
   * @param n le nom de la nouvelle feuille
   */
  Feuille(std::string n);




  /** Méthode getCellule
   * @param r la référence de la cellule
   * @return un pointeur sur la cellule ou nullptr lorsque la cellule de référence r n'existe
   * pas dans cette feuille.
   */
  Cellule * getCellule(const Reference r);

  /** méthode affectant une cellule de la Map, après avoir éventuellement détruit l'ancienne cellule
   * @param t un ptr sur la référence absolue
   * @param s un ptr sur la chaîne de saisie
   * param e un ptr sur l'expression arborescente
   */
  void setCellule(Reference *r, string * s, Expression *e) ;

  virtual ~Feuille() {
  }

  /** accesseur du nom d'une feuille
   *
   */
  const std::string& getNom() const {
    return nom;
  }
  bool operator!=(const Feuille& d) {
    return nom!=d.nom;
  }
};
// fin de la classe Feuille

} /* namespace modele */

#endif /* FEUILLE_H_ */
