/*
 * Plage.h
 *
 *  Created on: 16 avr. 2015
 *      Author: michel
 */

#ifndef PLAGE_H_
#define PLAGE_H_

#include <iostream>     // std::cout
#include <string>

#include "Expression.h"
#include "Feuille.h"
#include "Reference.h"

//#include "PlageIterator.h"

namespace modele {
class Feuille;

class Plage: public Argument {
protected:
  Feuille* feuille;
  Ref *gauche;
  Ref *droite;
  Reference ga;  // référence absolue de gauche
  Reference da;  // référence absolue de droite

public:
  class Iterator;   // déclaration en avant de a classe imbriquée
  Plage(Feuille *f,Ref *rg, Ref *rd) :  feuille(f), gauche(rg), droite(rd), ga(rg->toAbsolue()), da(rd->toAbsolue()) {
//    if (dynamic_cast<Reference *>(rg)){
//      ga=Reference(*rg);
//    } else{
//      ga=rg->toAbsolue();
//    if (dynamic_cast<Reference *>(rd)){
//       da=Reference(*rd);
//     } else{
 //   da=rd->toAbsolue();
  }

  bool estCalculable(){ return false; }
  /** Clonage d'une plage
   * @return une copie dynamique (new) de la plage.
   */
  Argument *clone(){
    Argument * pa=new Plage(feuille, gauche, droite);
    return pa;
  }
  std::string toString(){
    return std::string(gauche->toString()+std::string(":")+droite->toString());
  }

  /** Lors du collage du presse-papier, les références relatives doivent être mises à jour en fonction de la référence absolue
   * de la cellule destination. Les références absolues ga et da doivent l'être également.
   */
  void majRefRelatives(Reference *r){
    gauche->majRefRelatives(r);
    droite->majRefRelatives(r);
    ga=gauche->toAbsolue();
    da=droite->toAbsolue();
  }
  /** retourne un itérateur de plage pointant sur la première référence de la plage (à gauche du :).
   *
   */
  Iterator begin(){
    return Iterator(this);
  }
  /** retourne un itérateur de plage pointant sur la position après la dernière référence valide de la plage (à droite du :).
   *
   */
  Iterator end(){
    Plage::Iterator it(this);
    it.vaALaFin();
    return it;
  }
  virtual ~Plage() {
    delete gauche;
    delete droite;
  }

  const Reference& getDa() const {
    return da;
  }

  const Reference& getGa() const {
    return ga;
  }


  /** Itérateur de plage permettant de parcourir l'ensemble des références absolues désignées par une Plage. Classe imbriquée de Plage.
   *
   */
  class Iterator { // : public std::iterator<std::input_iterator_tag, Reference>
  protected:
    Plage *pplage; // pointeur sur la plage à partir de laquelle l'itérateur est construit.
    Reference courante; // référence absolue courante
    int incColonne;
    int incLigne;
  public:
    /** Constructeur d'un itérateur de plage : la position courante est initialisée sur la référence absolue de gauche de la plage
     * dont un pointeur est passé en paramètre.
     *
     */
    Iterator(Plage *p) : pplage(p),courante(p->getGa()),incColonne(0),incLigne(0) {
      int diffligne=p->getDa().getLigne() - p->getGa().getLigne();  // droite - gauche
      if (diffligne<0)
        incLigne=-1;
      else
        incLigne=+1;      // else +1 si une seule ligne, il faut quand même avancer pour aller à la fin (itérateur)

      int diffcol=p->getDa().getColonne() - p->getGa().getColonne();  // droite - gauche
       if (diffcol<0)
         incColonne=-1;
       else if (diffcol>0)
         incColonne=+1;      // else 0 initialisé!
    }
    /** positionne l'itérateur (référence courante) sur la fin : fin(A1:B2)=A3
     *
     */
    void vaALaFin(){
      courante.setColonne(pplage->getGa().getColonne());
      courante.setLigne(pplage->getDa().getLigne()+incLigne);
    }
    /** constructeur par recopie
     *
     */
    Iterator(const Iterator& it) : pplage(it.pplage), courante(it.courante), incColonne(it.incColonne), incLigne(it.incLigne){}

    /** Opérateur de pré-incrémentation d'un itérateur de plage : on avance d'abord sur la ligne courante à la colonne précédente ou
     * suivante (selon l'incrément : a3:c5 ou c5:a3). Quand on ne peut plus avancer sur la ligne courante alors on incrémente ou décrémente
     * la ligne courante. On termine sur la colonne de gauche et sur la ligne de droite + incLigne.
     */
    Iterator& operator++() {
      if(courante.getColonne()==pplage->getGa().getColonne()&&courante.getLigne()==pplage->getDa().getLigne()+incLigne){
        // si on est au bout
        return *this;                                                   // ne plus rien faire
      } else if (courante.getColonne()!=pplage->getDa().getColonne()){  // si on peut avancer sur la colonne
        courante.setColonne(courante.getColonne()+incColonne);  // on avance et on retourne !
        return *this;
      } else {
        courante.setColonne(pplage->getGa().getColonne());      // sinon on revient sur la colonne de gauche
        courante.setLigne(courante.getLigne()+incLigne);        // et on avance d'une ligne
        return *this;
        // donc le end d'un itérateur est la référence de colonne Ga.col et de ligne Da.ligne+incLigne
      } //      else if(courante.getLigne()!=pplage->getDa().getLigne())
    }
    /** Opérateur de post-incrémentation d'un itérateur de plage : on avance d'abord sur la ligne courante à la colonne précédente ou
      * suivante (selon l'incrément : a3:c5 ou c5:a3). Quand on ne plus avancer sur la ligne courante alors on incrémente ou décrémente
      * la ligne courante
      */
     Iterator operator++(int) {
       Iterator tmp(*this); operator++(); return tmp;
     }

//    bool operator==(const Iterator& d) {
//      return *pplage==*(d.pplage) && courante==d.courante;
//    }
     /** Opérateur "différent de" vérifiant que les deux références courantes de sont différentes
      *
      */
    bool operator!=(const Iterator& d) {return courante!=d.courante;}
    /** Déréférencement de l'itérateur qui pointe sur une référence absolue
     * @return une référence (C++) sur la Reference courante
     */
    Reference& operator*() {return courante;}
  };

};

} /* namespace modele */


#endif /* PLAGE_H_ */
