/*
 * Reference.h
 *
 *  Created on: 26 mars 2015
 *      Author: michel
 */

#ifndef REFERENCE_H_
#define REFERENCE_H_

#include "Expression.h"   // Reference héritant de Expression, il faut sa définition complète et non une forward déclar
//#include "Feuille.h"
#include "Cellule.h"
#include "Ref.h"

namespace modele {
//class Expression;
class Feuille;
class Valeur;
/** Référence absolue de cellule définie par un numéro de ligne [1..16384], un numéro de colonne [1..230] et un pointeur sur
 * la feuille. Attention la cellule correspondante peut exister ou non dans la feuille pointée dans  :
 * map<Reference,Cellule> feuille->cellules.
 */
class Reference: public Ref {

protected:
  int ligne;  // comprise entre 1..n
  int colonne;  // comprise entre 1..n
  Feuille * feuille; // référence à la feuille

public:

  /** numéro maxi de ligne (mini=1)  */
  static const int MAXLIGNE = 16384; // idem Excel !
  /** numéro maxi de colonne (mini=1)  */
  static const int MAXCOLONNE = 230; // idem Excel !

  /** Constructeur créant une Reference.
   * @param lig le numéro de ligne compris entre 1 et MAXLIGNE
   * @param col le numéro de colonne compris entre 1 et MAXCOLONNE
   * @param f la feuille
   * @exception Exception si les indices dépassent !
   */
  Reference(int lig, int col, Feuille * f);

//  /** Constructeur créant une Reference par défaut.
//   * @param f la feuille
//   * @exception Exception si les indices dépassent !
//   */
//  Reference():feuille(), ligne(1), colonne(1) {}

  modele::Valeur * calculer();

  /** Opérateur de comparaison de deux références. On pourra ainsi les utiliser comme clé dans une map
   *
   */
  bool const operator<(const Reference & d) const;

  std::string toString();

  virtual ~Reference() {}

  Feuille* getFeuille() const {
    return feuille;
  }

  void setFeuille(Feuille* feuille) {
    this->feuille = feuille;
  }

  int getColonne() const {
    return colonne;
  }

  int getLigne() const {
    return ligne;
  }
  /** Tente d'accéder à la cellule correspondant à cette Reference.
   * @return un ptr sur la Cellule recherchée si celle-ci existe, nullptr sinon
   */
  Cellule* getCellule() const ;

  Expression *clone(){
    return new Reference(*this);
  }
  Reference toAbsolue() {
    return *this;
  }

  void setColonne(int colonne) {
    this->colonne = colonne;
  }

  void setLigne(int ligne) {
    this->ligne = ligne;
  }
  bool operator!=(const Reference& d) {
    return feuille!=d.feuille || ligne!=d.ligne || colonne!=d.colonne;
  }
};

} /* namespace modele */

#endif /* REFERENCE_H_ */
