/*
 * Sinus.h
 *
 *  Created on: 8 avr. 2015
 *      Author: michel
 */

#ifndef SINUS_H_
#define SINUS_H_

#include <cmath>    // sin
#include "ValChaine.h"
#include "ValErreur.h"
#include "ValDouble.h"
#include "ValVide.h"

#include "Fonction.h"

namespace modele {
/** La classe Sinus permet d'instancier des expressions fonctionnelles telles que : sin(3.14), sin(A1+2), ...
 * Elle hérite de la classe Fonction qui gère la liste des arguments de chaque fonction. Dans le cas de sinus, il
 * n'y a qu'un seul argument obligatoire dont la valeur est exprimée en radian.
 */
class Sinus: public Fonction {
protected:
  /** nom de la fonction : SIN
    *
    */
   static const std::string nom;  //="SIN"; PAS INLINE

   /** Nombre de paramètres de la fonction :
     * -1 : illimité
     * 0 : aucun
     * 1..n : n paramètres
     */
   static const int nbParam=1;  // comme c'est const, on peut inline

public:

  /** constructeur d'une instance de fonction sinus sur une expression
   * @param e un ptr sur l'expression dont il faut calculer le sinus
   */
  Sinus(Expression *e){
    listeParam.push_back(e);   // construit la liste des paramètres
  }
  /** calcule la valeur de l'expression sinus(e)
   * @returns la Valeur de cette expression sin(e)
   */
  modele::Valeur * calculer(){
//    double dg=0;
//    Argument *a=listeParam.front();
//    if(!a->estCalculable()){
//        return new ValErreur("#PLAGE");
//    }
    Expression* pe;
    if((pe=dynamic_cast<Expression*>(listeParam.front()))){
      Valeur * vg=pe->calculer(); // nouvelle valeur calculée sur l'expression
      if(dynamic_cast<ValErreur*>(vg)){ // si erreur à gauche !
        //cout<<"DEBUG:ERREUR:vg:"<<vg<<endl;
        return vg;            // propagation
      } else if(dynamic_cast<ValChaine*>(vg)||dynamic_cast<ValVide*>(vg)){ // si chaîne ou vide
        delete vg;  // désalocation
        return new ValErreur("#VALEUR");  // nouvelle
      } else if(ValDouble* vdg=dynamic_cast<ValDouble*>(vg)){ // si double
        //cout<<"DEBUG:vdg:"<<vdg<<endl;
        double dg=vdg->getDouble();
        delete vg;
        return new ValDouble(std::sin(dg));
      } //else cout<<"DEBUG:Aucun dynamic cast!"<<endl;
    } else {
      return new ValErreur("#ARGUMENT");
    }
  }

  /** Fonction retournant une chaîne représentant la fonction
   * @return une string de la forme : SIN(A1+1)
   */
  std::string toString(){
    std::string r=Sinus::nom+"(";
    bool premiereFois=true;  // pour les ;
    for (Argument *e: listeParam){
      if (premiereFois) {
        premiereFois=false; // ce n'est plus la premiere fois !
      } else {
        r+="; ";
      }
      r+=e->toString();
    }
    return r+")";
  }
  /** Clonage arborescent d'une expression sinus. Indispensable pour la fonctionalité du COLLER (paste).
   * @returns un pointeur sur une nouvelle Expression copie de l'originale.
   */
  Expression *clone(){
    return new Sinus(dynamic_cast<Expression *>(listeParam.front())->clone()); // pas de sinus de Plage
  }
  ~Sinus() {
    // TODO Auto-generated destructor stub
  }
};

} /* namespace modele */

#endif /* SINUS_H_ */
