/*
 * Somme.h
 *
 *  Created on: 9 avr. 2015
 *      Author: michel
 */

#ifndef SOMME_H_
#define SOMME_H_
#include <list>
#include "Fonction.h"
#include "ValVide.h"
#include "ValChaine.h"
#include "ValErreur.h"
#include "ValDouble.h"
#include "Plage.h"

namespace modele {
class Argument;
class Expression;


class Somme: public Fonction {
protected:
  /** nom de la fonction : SOMME
   *
   */
  static std::string nom;

  /** Nombre de paramètres de la fonction :
   * -1 : illimité
   * 0 : aucun
   * 1..n : n paramètres
   */
   static int nbParam;  // par défaut illimité

public:
  Somme(std::list<Argument *> larg): Fonction(larg) {}

  Valeur *calculer(){
    double d=0;
    for(Argument *pa : listeParam){ // pour chaque argument
      if(Expression* e=dynamic_cast<Expression*>(pa)){ // traitement d'une expression
        Valeur *v=e->calculer();
        if(dynamic_cast<ValErreur*>(v)){ // si erreur
          //cout<<"DEBUG:ERREUR:v:"<<v<<endl;
          return v;            // propagation
        } else if(dynamic_cast<ValChaine*>(v)){ // si chaîne à gauche !
          delete v;  // désalocation et d=d+0 chaine==0
        } else if(dynamic_cast<ValVide*>(v)){ // si vide à gauche !
          delete v;    // d est déjà égal à 0;
        } else if(ValDouble* vd=dynamic_cast<ValDouble*>(v)){ // si double à gauche !
          //cout<<"DEBUG:vdg:"<<vdg<<endl;
          d+=vd->getDouble();
          delete v;
        } //else cout<<"DEBUG:Aucun dynamic cast!"<<endl;
      } else if(Plage* p=dynamic_cast<Plage*>(pa)){     // traitement d'une plage
        for(Plage::Iterator it=p->begin();it!=p->end();it++){
          // std::cout<<"DEBUG:courante: "<<(*it).toString()<<std::endl;
          Valeur *v=(*it).calculer();
          if(dynamic_cast<ValErreur*>(v)){ // si erreur
            //cout<<"DEBUG:ERREUR:v:"<<v<<endl;
            return v;            // propagation
          } else if(dynamic_cast<ValChaine*>(v)){ // si chaîne à gauche !
            delete v;  // désalocation et d=d+0 chaine==0
          } else if(dynamic_cast<ValVide*>(v)){ // si vide à gauche !
            delete v;    // d est déjà égal à 0;
          } else if(ValDouble* vd=dynamic_cast<ValDouble*>(v)){ // si double à gauche !
            //cout<<"DEBUG:vdg:"<<vdg<<endl;
            d+=vd->getDouble();
            delete v;
          } //else cout<<"DEBUG:Aucun dynamic cast!"<<endl;
        } // fin for
      } // fin if plage
    } // fin du for param
    return new ValDouble(d);
  }

  /** Fonction retournant une chaîne représentant la fonction
   * @return la fonction et ses arguments
   */
  std::string toString(){
    std::string r=Somme::nom+"(";
    bool premiereFois=true;  // pour les ;
    for (Argument *e: listeParam){
      if (premiereFois) {
        premiereFois=false; // ce n'est plus la premiere fois !
      } else {
        r+="; ";
      }
      r+=e->toString();
    }
    return r+")";
  }

  Expression *clone(){
    return new Somme(std::list<Argument *>(listeParam));
  }

  ~Somme() {
    // TODO Auto-generated destructor stub
  }
};

} /* namespace modele */

#endif /* SOMME_H_ */
